@@ -384,7 +384,7 @@ char *categoryPathname[] =
 
 	if ((all == nil) && (u == nil)) return nil;
 
-	item = [[LUDictionary alloc] init];
+	item = [[LUDictionary alloc] initTimeStamped];
 	[item setValue:name forKey:"name"];
 
 	if (u != nil)
@@ -417,7 +417,7 @@ char *categoryPathname[] =
 
 	if (name == NULL) return nil;
 
-	item = [[LUDictionary alloc] init];
+	item = [[LUDictionary alloc] initTimeStamped];
 	[item setValue:name forKey:"name"];
 	found = NO;
 
